@@ -5,9 +5,9 @@ var writeToStream = require('./writeToStream')
 var EE = require('events').EventEmitter
 var inherits = require('inherits')
 
-function generate (packet) {
+function generate (packet, opts) {
   var stream = new Accumulator()
-  writeToStream(packet, stream)
+  writeToStream(packet, stream, opts)
   return stream.concat()
 }
 
@@ -31,7 +31,7 @@ Accumulator.prototype.concat = function () {
   var i
   var result
 
-  for (i = 0; i < list.length && list[i]; i++) {
+  for (i = 0; i < list.length && list[i] !== undefined; i++) {
     if (typeof list[i] !== 'string') lengths[i] = list[i].length
     else lengths[i] = Buffer.byteLength(list[i])
 
@@ -40,7 +40,7 @@ Accumulator.prototype.concat = function () {
 
   result = Buffer.allocUnsafe(length)
 
-  for (i = 0; i < list.length && list[i]; i++) {
+  for (i = 0; i < list.length && list[i] !== undefined; i++) {
     if (typeof list[i] !== 'string') {
       list[i].copy(result, pos)
       pos += lengths[i]
