@@ -1088,6 +1088,20 @@ testParseGenerate('puback', {
   0, 2 // Message ID
 ]))
 
+testParseGenerate('puback with reason and no MQTT5 properties', {
+  cmd: 'puback',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 3,
+  messageId: 2,
+  reasonCode: 16
+}, Buffer.from([
+  64, 3, // Header
+  0, 2, // Message ID
+  16 // reason code
+]), {protocolVersion: 5})
+
 testParseGenerate('puback MQTT5 properties', {
   cmd: 'puback',
   retain: false,
@@ -1770,6 +1784,75 @@ testGenerateError('Username is required to use password', {
   password: 'password'
 })
 
+testGenerateError('Invalid messageExpiryInterval: -4321', {
+  cmd: 'publish',
+  retain: true,
+  qos: 2,
+  dup: true,
+  length: 60,
+  topic: 'test',
+  payload: new Buffer('test'),
+  messageId: 10,
+  properties: {
+    payloadFormatIndicator: true,
+    messageExpiryInterval: -4321,
+    topicAlias: 100,
+    responseTopic: 'topic',
+    correlationData: Buffer.from([1, 2, 3, 4]),
+    userProperties: {
+      'test': 'test'
+    },
+    subscriptionIdentifier: 120,
+    contentType: 'test'
+  }
+}, {protocolVersion: 5})
+
+testGenerateError('Invalid topicAlias: -100', {
+  cmd: 'publish',
+  retain: true,
+  qos: 2,
+  dup: true,
+  length: 60,
+  topic: 'test',
+  payload: new Buffer('test'),
+  messageId: 10,
+  properties: {
+    payloadFormatIndicator: true,
+    messageExpiryInterval: 4321,
+    topicAlias: -100,
+    responseTopic: 'topic',
+    correlationData: Buffer.from([1, 2, 3, 4]),
+    userProperties: {
+      'test': 'test'
+    },
+    subscriptionIdentifier: 120,
+    contentType: 'test'
+  }
+}, {protocolVersion: 5})
+
+testGenerateError('Invalid subscriptionIdentifier: -120', {
+  cmd: 'publish',
+  retain: true,
+  qos: 2,
+  dup: true,
+  length: 60,
+  topic: 'test',
+  payload: new Buffer('test'),
+  messageId: 10,
+  properties: {
+    payloadFormatIndicator: true,
+    messageExpiryInterval: 4321,
+    topicAlias: 100,
+    responseTopic: 'topic',
+    correlationData: Buffer.from([1, 2, 3, 4]),
+    userProperties: {
+      'test': 'test'
+    },
+    subscriptionIdentifier: -120,
+    contentType: 'test'
+  }
+}, {protocolVersion: 5})
+
 test('support cork', function (t) {
   t.plan(9)
 
