@@ -6,6 +6,7 @@ var empty = Buffer.allocUnsafe(0)
 var zeroBuf = Buffer.from([0])
 var numbers = require('./numbers')
 var nextTick = require('process-nextick-args').nextTick
+var debug = require('debug')('mqtt-packet:writeToStream')
 
 var numCache = numbers.cache
 var generateNumber = numbers.generateNumber
@@ -16,6 +17,7 @@ var writeNumber = writeNumberCached
 var toGenerate = true
 
 function generate (packet, stream, opts) {
+  debug('generate called')
   if (stream.cork) {
     stream.cork()
     nextTick(uncork, stream)
@@ -25,7 +27,7 @@ function generate (packet, stream, opts) {
     toGenerate = false
     generateCache()
   }
-
+  debug('generate: packet.cmd: %s', packet.cmd)
   switch (packet.cmd) {
     case 'connect':
       return connect(packet, stream, opts)
@@ -141,6 +143,7 @@ function connect (packet, stream, opts) {
   // Properties
   if (protocolVersion === 5) {
     var propertiesData = getProperties(stream, properties)
+    if (!propertiesData) { return false }
     length += propertiesData.length
   }
 
@@ -177,6 +180,7 @@ function connect (packet, stream, opts) {
     var willProperties = {}
     if (protocolVersion === 5) {
       willProperties = getProperties(stream, will.properties)
+      if (!willProperties) { return false }
       length += willProperties.length
     }
   }
@@ -283,6 +287,7 @@ function connack (packet, stream, opts) {
   var propertiesData = null
   if (version === 5) {
     propertiesData = getProperties(stream, properties)
+    if (!propertiesData) { return false }
     length += propertiesData.length
   }
 
@@ -299,6 +304,7 @@ function connack (packet, stream, opts) {
 }
 
 function publish (packet, stream, opts) {
+  debug('publish: packet: %o', packet)
   var version = opts ? opts.protocolVersion : 4
   var settings = packet || {}
   var qos = settings.qos || 0
@@ -332,6 +338,7 @@ function publish (packet, stream, opts) {
   var propertiesData = null
   if (version === 5) {
     propertiesData = getProperties(stream, properties)
+    if (!propertiesData) { return false }
     length += propertiesData.length
   }
 
@@ -354,6 +361,7 @@ function publish (packet, stream, opts) {
   }
 
   // Payload
+  debug('publish: payload: %o', payload)
   return stream.write(payload)
 }
 
@@ -380,9 +388,12 @@ function confirmation (packet, stream, opts) {
   // properies mqtt 5
   var propertiesData = null
   if (version === 5) {
-    propertiesData = getPropertiesByMaximumPacketSize(stream, properties, opts, length)
-    if (!propertiesData) { return false }
-    length += propertiesData.length
+    // Confirm should not add empty property length with no properties (rfc 3.4.2.2.1)
+    if (typeof properties === 'object') {
+      propertiesData = getPropertiesByMaximumPacketSize(stream, properties, opts, length)
+      if (!propertiesData) { return false }
+      length += propertiesData.length
+    }
   }
 
   // Header
@@ -407,6 +418,7 @@ function confirmation (packet, stream, opts) {
 }
 
 function subscribe (packet, stream, opts) {
+  debug('subscribe: packet: ')
   var version = opts ? opts.protocolVersion : 4
   var settings = packet || {}
   var dup = settings.dup ? protocol.DUP_MASK : 0
@@ -426,6 +438,7 @@ function subscribe (packet, stream, opts) {
   var propertiesData = null
   if (version === 5) {
     propertiesData = getProperties(stream, properties)
+    if (!propertiesData) { return false }
     length += propertiesData.length
   }
 
@@ -470,6 +483,7 @@ function subscribe (packet, stream, opts) {
   }
 
   // Generate header
+  debug('subscribe: writing to stream: %o', protocol.SUBSCRIBE_HEADER)
   stream.write(protocol.SUBSCRIBE_HEADER[1][dup ? 1 : 0][0])
 
   // Generate length
@@ -599,6 +613,7 @@ function unsubscribe (packet, stream, opts) {
   var propertiesData = null
   if (version === 5) {
     propertiesData = getProperties(stream, properties)
+    if (!propertiesData) { return false }
     length += propertiesData.length
   }
 
@@ -775,7 +790,7 @@ function writeVarByteInt (stream, num) {
     buffer = genBufVariableByteInt(num).data
     if (num < 16384) varByteIntCache[num] = buffer
   }
-
+  debug('writeVarByteInt: writing to stream: %o', buffer)
   stream.write(buffer)
 }
 
@@ -794,6 +809,7 @@ function writeString (stream, string) {
   var strlen = Buffer.byteLength(string)
   writeNumber(stream, strlen)
 
+  debug('writeString: %s', string)
   return stream.write(string, 'utf8')
 }
 
@@ -823,13 +839,19 @@ function writeStringPair (stream, name, value) {
  * @api private
  */
 function writeNumberCached (stream, number) {
+  debug('writeNumberCached: number: %d', number)
+  debug('writeNumberCached: %o', numCache[number])
   return stream.write(numCache[number])
 }
 function writeNumberGenerated (stream, number) {
-  return stream.write(generateNumber(number))
+  var generatedNumber = generateNumber(number)
+  debug('writeNumberGenerated: %o', generatedNumber)
+  return stream.write(generatedNumber)
 }
 function write4ByteNumber (stream, number) {
-  return stream.write(generate4ByteBuffer(number))
+  var generated4ByteBuffer = generate4ByteBuffer(number)
+  debug('write4ByteNumber: %o', generated4ByteBuffer)
+  return stream.write(generated4ByteBuffer)
 }
 /**
  * writeStringOrBuffer - write a String or Buffer with the its length prefix
@@ -865,15 +887,15 @@ function getProperties (stream, properties) {
     switch (type) {
       case 'byte': {
         if (typeof value !== 'boolean') {
-          stream.emit('error', new Error('Invalid ' + name))
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + 1
         break
       }
       case 'int8': {
-        if (typeof value !== 'number') {
-          stream.emit('error', new Error('Invalid ' + name))
+        if (typeof value !== 'number' || value < 0 || value > 0xff) {
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + 1
@@ -881,31 +903,31 @@ function getProperties (stream, properties) {
       }
       case 'binary': {
         if (value && value === null) {
-          stream.emit('error', new Error('Invalid ' + name))
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + Buffer.byteLength(value) + 2
         break
       }
       case 'int16': {
-        if (typeof value !== 'number') {
-          stream.emit('error', new Error('Invalid ' + name))
+        if (typeof value !== 'number' || value < 0 || value > 0xffff) {
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + 2
         break
       }
       case 'int32': {
-        if (typeof value !== 'number') {
-          stream.emit('error', new Error('Invalid ' + name))
+        if (typeof value !== 'number' || value < 0 || value > 0xffffffff) {
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + 4
         break
       }
       case 'var': {
-        if (typeof value !== 'number') {
-          stream.emit('error', new Error('Invalid ' + name))
+        if (typeof value !== 'number' || value < 0 || value > 0xffffffff) {
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + genBufVariableByteInt(value).length
@@ -913,7 +935,7 @@ function getProperties (stream, properties) {
       }
       case 'string': {
         if (typeof value !== 'string') {
-          stream.emit('error', new Error('Invalid ' + name))
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += 1 + 2 + Buffer.byteLength(value.toString())
@@ -921,7 +943,7 @@ function getProperties (stream, properties) {
       }
       case 'pair': {
         if (typeof value !== 'object') {
-          stream.emit('error', new Error('Invalid ' + name))
+          stream.emit('error', new Error('Invalid ' + name + ': ' + value))
           return false
         }
         length += Object.getOwnPropertyNames(value).reduce(function (result, name) {
@@ -939,7 +961,7 @@ function getProperties (stream, properties) {
         break
       }
       default: {
-        stream.emit('error', new Error('Invalid property ' + name))
+        stream.emit('error', new Error('Invalid property ' + name + ': ' + value))
         return false
       }
     }
@@ -948,13 +970,18 @@ function getProperties (stream, properties) {
   if (properties) {
     for (var propName in properties) {
       var propLength = 0
+      var propValueLength = 0
       var propValue = properties[propName]
       if (Array.isArray(propValue)) {
         for (var valueIndex = 0; valueIndex < propValue.length; valueIndex++) {
-          propLength += getLengthProperty(propName, propValue[valueIndex])
+          propValueLength = getLengthProperty(propName, propValue[valueIndex])
+          if (!propValueLength) { return false }
+          propLength += propValueLength
         }
       } else {
-        propLength = getLengthProperty(propName, propValue)
+        propValueLength = getLengthProperty(propName, propValue)
+        if (!propValueLength) { return false }
+        propLength = propValueLength
       }
       if (!propLength) return false
       propertiesLength += propLength
